@@ -1,29 +1,26 @@
-/*
- * AncestorDialog.cpp is part of Brewtarget, and is Copyright the following
- * authors 2016-2021
- * - Matt Young <mfsy@yahoo.com>
- * - Mik Firestone <mikfire@fastmail.com>
+/*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
+ * AncestorDialog.cpp is part of Brewtarget, and is copyright the following authors 2021-2024:
+ *   • Matt Young <mfsy@yahoo.com>
+ *   • Mik Firestone <mikfire@fastmail.com>
  *
- * Brewtarget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+ * version.
  *
- * Brewtarget is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+ * You should have received a copy of the GNU General Public License along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌*/
 #include "AncestorDialog.h"
 
 #include <algorithm>
 
+#include <QAction>
 #include <QDebug>
 #include <QtCore/QVariant>
-#include <QtWidgets/QAction>
 #include <QtWidgets/QApplication>
 #include <QtWidgets/QButtonGroup>
 #include <QtWidgets/QComboBox>
@@ -40,6 +37,11 @@
 #include "model/NamedEntity.h"
 #include "model/Recipe.h"
 
+#ifdef BUILDING_WITH_CMAKE
+   // Explicitly doing this include reduces potential problems with AUTOMOC when compiling with CMake
+   #include "moc_AncestorDialog.cpp"
+#endif
+
 AncestorDialog::AncestorDialog(QWidget * parent) : QDialog(parent) {
 
    setupUi(this);
@@ -73,7 +75,7 @@ void AncestorDialog::buildAncestorBox() {
    QList<Recipe *> recipes = ObjectStoreWrapper::getAllRaw<Recipe>();
    std::sort(recipes.begin(), recipes.end(), AncestorDialog::recipeLessThan);
 
-   foreach (Recipe * recipe, recipes) {
+   for (auto recipe : recipes) {
       if (recipe->display()) {
          comboBox_ancestor->addItem(recipe->name(), recipe->key());
       }
@@ -87,7 +89,7 @@ void AncestorDialog::buildDescendantBox(Recipe * ignore) {
    std::sort(recipes.begin(), recipes.end(), recipeLessThan);
 
    //  The rules of what can be a target are complex
-   foreach (Recipe * recipe, recipes) {
+   for (auto recipe : recipes) {
       // if we are ignoring the recipe, skip
       if (recipe == ignore) {
          continue;
