@@ -1,36 +1,49 @@
-/*
- * Localization.h is part of Brewtarget, and is copyright the following
- * authors 2011-2023
- * - Greg Meess <Daedalus12@gmail.com>
- * - Matt Young <mfsy@yahoo.com>
- * - Mik Firestone <mikfire@gmail.com>
- * - Philip Greggory Lee <rocketman768@gmail.com>
+/*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
+ * Localization.h is part of Brewtarget, and is copyright the following authors 2011-2024:
+ *   • Greg Meess <Daedalus12@gmail.com>
+ *   • Matt Young <mfsy@yahoo.com>
+ *   • Mik Firestone <mikfire@gmail.com>
+ *   • Philip Greggory Lee <rocketman768@gmail.com>
  *
- * Brewtarget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+ * version.
  *
- * Brewtarget is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+ * You should have received a copy of the GNU General Public License along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌*/
 #ifndef LOCALIZATION_H
 #define LOCALIZATION_H
 #pragma once
 
 #include <QDate>
+#include <QIcon>
 #include <QLocale>
 #include <QString>
+#include <QVector>
 
 class BtStringConst;
 class NamedEntity;
 
 namespace Localization {
+   struct LanguageInfo {
+      int          qtLanguageCode;     // From enum QLocale::Language
+      QString      iso639_1Code;       // The two-letter language code used by Localization::setLanguage()
+      QIcon        countryFlag;        // Yes, we know some languages are spoken in more than one country...
+      char const * nameInEnglish;
+      QString      nameInDefaultLang;  // Don't strictly need to store this, but having the hard-coded tr() calls
+                                       // in the initialisation flag up what language names need translating
+   };
+
+   /**
+    * \brief Returns a reference to the list holding info about the languages we support
+    */
+   QVector<LanguageInfo> const & languageInfo();
+
    /**
     * \brief Returns the locale to use for formatting numbers etc.  Usually this is the same as \c QLocale::system(),
     *        but can be overridden for testing purposes.
@@ -81,6 +94,13 @@ namespace Localization {
     */
    QString displayDateUserFormated(QDate const & date);
 
+   /**
+    * \return \c true if the supplied language code is one we support, \c false otherwise
+    */
+   [[nodiscard]] bool isSupportedLanguage(QString const & twoLetterLanguage);
+
+   void setLanguage(QLocale const & newLanguageLocale);
+
    /**
     * \brief Loads the translator with two letter ISO 639-1 code.
     *
@@ -89,19 +109,19 @@ namespace Localization {
     *
     * \param twoLetterLanguage two letter ISO 639-1 code
     */
-   void setLanguage(QString twoLetterLanguage);
+   void setLanguage(QString const & twoLetterLanguage);
 
    /**
     * \brief Gets the 2-letter ISO 639-1 language code we are currently using.
     * \returns current 2-letter ISO 639-1 language code.
     */
-   QString const & getCurrentLanguage();
+   QString getCurrentLanguage();
 
    /**
     * \brief Gets the ISO 639-1 language code for the system.
     * \returns current 2-letter ISO 639-1 system language code
     */
-   QString const & getSystemLanguage();
+   QString getSystemLanguage();
 
    /**
     * \brief Convert a \c QString to a \c double, if possible using the default locale and if not using the C locale.
@@ -152,17 +172,17 @@ namespace Localization {
    bool hasUnits(QString qstr);
 
    /**
-    * \brief Load translation files.
+    * \brief Load default translation file if necessary (usually only the first time ever the software is run).
     */
    void loadTranslations();
 
    /**
-    * \brief Load localization settings from persistent storage
+    * \brief Load localization settings from persistent storage, including the language setting from the previous run.
     */
    void loadSettings();
 
    /**
-    * \brief Save localization settings to persistent storage
+    * \brief Save localization settings to persistent storage, including current language setting.
     */
    void saveSettings();
 }
