@@ -1,24 +1,29 @@
-/*
- * OptionDialog.cpp is part of Brewtarget, and is Copyright the following
- * authors 2009-2023
- * - Matt Young <mfsy@yahoo.com>
- * - Mik Firestone <mikfire@gmail.com>
- * - Philip Greggory Lee <rocketman768@gmail.com>
- * - Rob Taylor <robtaylor@floopily.org>
+/*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
+ * OptionDialog.cpp is part of Brewtarget, and is copyright the following authors 2009-2024:
+ *   • Brian Rower <brian.rower@gmail.com>
+ *   • Daniel Pettersson <pettson81@gmail.com>
+ *   • Greg Meess <Daedalus12@gmail.com>
+ *   • Idar Lund <idarlund@gmail.com>
+ *   • Mark de Wever <koraq@xs4all.nl>
+ *   • Mattias Måhl <mattias@kejsarsten.com>
+ *   • Matt Young <mfsy@yahoo.com>
+ *   • Maxime Lavigne <duguigne@gmail.com>
+ *   • Mik Firestone <mikfire@gmail.com>
+ *   • Philip Greggory Lee <rocketman768@gmail.com>
+ *   • Rob Taylor <robtaylor@floopily.org>
+ *   • Théophane Martin <theophane.m@gmail.com>
  *
- * Brewtarget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+ * version.
  *
- * Brewtarget is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+ * You should have received a copy of the GNU General Public License along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌*/
 #include "OptionDialog.h"
 
 #include <optional>
@@ -35,6 +40,7 @@
 #include <QVector>
 #include <QWidget>
 
+#include "config.h"
 #include "database/Database.h"
 #include "Localization.h"
 #include "Logging.h"
@@ -46,6 +52,11 @@
 #include "measurement/UnitSystem.h"
 #include "PersistentSettings.h"
 
+#ifdef BUILDING_WITH_CMAKE
+   // Explicitly doing this include reduces potential problems with AUTOMOC when compiling with CMake
+   #include "moc_OptionDialog.cpp"
+#endif
+
 //
 // Anonymous namespace for constants, global variables and functions used only in this file
 //
@@ -58,14 +69,6 @@ namespace {
       TEST_PASSED
    };
 
-   struct LanguageInfo {
-      QString            iso639_1Code;       // What we need to pass to Localization::setLanguage()
-      QIcon              countryFlag;        // Yes, we know some languages are spoken in more than one country...
-      char const    *    nameInEnglish;
-      QString            nameInCurrentLang;  // Don't strictly need to store this, but having the hard-coded tr() calls
-                                             // in the initialisation flag up what language names need translating
-   };
-
    /**
     * \brief For a given QComboBox, save the UnitSystem it has selected
     *
@@ -85,8 +88,7 @@ namespace {
       for (auto unitSystem : unitSystems) {
          if (selection == unitSystem->uniqueName) {
             qDebug() <<
-               Q_FUNC_INFO << "Setting UnitSystem for" << Measurement::getDisplayName(physicalQuantity) << "to" <<
-               unitSystem->uniqueName;
+               Q_FUNC_INFO << "Setting UnitSystem for" << physicalQuantity << "to" << unitSystem->uniqueName;
             Measurement::setDisplayUnitSystem(physicalQuantity, *unitSystem);
             return true;
          }
@@ -94,7 +96,7 @@ namespace {
 
       qWarning() <<
          Q_FUNC_INFO << "Unable to interpret value " << selection << "of" << comboBoxName << "as UnitSystem name for" <<
-         Measurement::getDisplayName(physicalQuantity);
+         physicalQuantity;
       return false;
    }
 
@@ -107,59 +109,32 @@ public:
    /**
     * Constructor
     */
-   impl(OptionDialog & optionDialog) :
-      qFileDialog                {&optionDialog},
-      label_pgHostname           {optionDialog.groupBox_dbConfig},
-      input_pgHostname           {optionDialog.groupBox_dbConfig},
-      label_pgPortNum            {optionDialog.groupBox_dbConfig},
-      input_pgPortNum            {optionDialog.groupBox_dbConfig},
-      label_pgSchema             {optionDialog.groupBox_dbConfig},
-      input_pgSchema             {optionDialog.groupBox_dbConfig},
-      label_pgDbName             {optionDialog.groupBox_dbConfig},
-      input_pgDbName             {optionDialog.groupBox_dbConfig},
-      label_pgUsername           {optionDialog.groupBox_dbConfig},
-      input_pgUsername           {optionDialog.groupBox_dbConfig},
-      label_pgPassword           {optionDialog.groupBox_dbConfig},
-      input_pgPassword           {optionDialog.groupBox_dbConfig},
-      checkBox_savePgPassword    {optionDialog.groupBox_dbConfig},
-      label_userDataDir          {optionDialog.groupBox_dbConfig},
-      input_userDataDir          {optionDialog.groupBox_dbConfig},
-      pushButton_browseDataDir   {optionDialog.groupBox_dbConfig},
-      label_backupDir            {optionDialog.groupBox_dbConfig},
-      input_backupDir            {optionDialog.groupBox_dbConfig},
-      pushButton_browseBackupDir {optionDialog.groupBox_dbConfig},
-      label_numBackups           {optionDialog.groupBox_dbConfig},
-      spinBox_numBackups         {optionDialog.groupBox_dbConfig},
-      label_frequency            {optionDialog.groupBox_dbConfig},
-      spinBox_frequency          {optionDialog.groupBox_dbConfig},
-      languageInfo {
-         //
-         // See also CmakeLists.txt for list of translation source files (in ../translations directory)
-         //
-         {"ca", QIcon(":images/flagCatalonia.svg"),   "Catalan",          tr("Catalan")          },
-         {"cs", QIcon(":images/flagCzech.svg"),       "Czech",            tr("Czech")            },
-         {"da", QIcon(":images/flagDenmark.svg"),     "Danish",           tr("Danish")           },
-         {"de", QIcon(":images/flagGermany.svg"),     "German",           tr("German")           },
-         {"el", QIcon(":images/flagGreece.svg"),      "Greek",            tr("Greek")            },
-         {"en", QIcon(":images/flagUK.svg"),          "English",          tr("English")          },
-         {"es", QIcon(":images/flagSpain.svg"),       "Spanish",          tr("Spanish")          },
-         {"et", QIcon(":images/flagEstonia.svg"),     "Estonian",         tr("Estonian")         },
-         {"eu", QIcon(":images/flagBasque.svg"),      "Basque",           tr("Basque")           },
-         {"fr", QIcon(":images/flagFrance.svg"),      "French",           tr("French")           },
-         {"gl", QIcon(":images/flagGalicia.svg"),     "Galician",         tr("Galician")         },
-         {"hu", QIcon(":images/flagHungary.svg"),     "Hungarian",        tr("Hungarian")        },
-         {"it", QIcon(":images/flagItaly.svg"),       "Italian",          tr("Italian")          },
-         {"lv", QIcon(":images/flagLatvia.svg"),      "Latvian",          tr("Latvian")          },
-         {"nb", QIcon(":images/flagNorway.svg"),      "Norwegian Bokmål", tr("Norwegian Bokmål") },
-         {"nl", QIcon(":images/flagNetherlands.svg"), "Dutch",            tr("Dutch")            },
-         {"pl", QIcon(":images/flagPoland.svg"),      "Polish",           tr("Polish")           },
-         {"pt", QIcon(":images/flagBrazil.svg"),      "Portuguese",       tr("Portuguese")       },
-         {"ru", QIcon(":images/flagRussia.svg"),      "Russian",          tr("Russian")          },
-         {"sr", QIcon(":images/flagSerbia.svg"),      "Serbian",          tr("Serbian")          },
-         {"sv", QIcon(":images/flagSweden.svg"),      "Swedish",          tr("Swedish")          },
-         {"tr", QIcon(":images/flagTurkey.svg"),      "Turkish",          tr("Turkish")          },
-         {"zh", QIcon(":images/flagChina.svg"),       "Chinese",          tr("Chinese")          }
-      } {
+   impl(OptionDialog & self) :
+      m_self                     {self},
+      qFileDialog                {&self},
+      label_pgHostname           {self.groupBox_dbConfig},
+      input_pgHostname           {self.groupBox_dbConfig},
+      label_pgPortNum            {self.groupBox_dbConfig},
+      input_pgPortNum            {self.groupBox_dbConfig},
+      label_pgSchema             {self.groupBox_dbConfig},
+      input_pgSchema             {self.groupBox_dbConfig},
+      label_pgDbName             {self.groupBox_dbConfig},
+      input_pgDbName             {self.groupBox_dbConfig},
+      label_pgUsername           {self.groupBox_dbConfig},
+      input_pgUsername           {self.groupBox_dbConfig},
+      label_pgPassword           {self.groupBox_dbConfig},
+      input_pgPassword           {self.groupBox_dbConfig},
+      checkBox_savePgPassword    {self.groupBox_dbConfig},
+      label_userDataDir          {self.groupBox_dbConfig},
+      input_userDataDir          {self.groupBox_dbConfig},
+      pushButton_browseDataDir   {self.groupBox_dbConfig},
+      label_backupDir            {self.groupBox_dbConfig},
+      input_backupDir            {self.groupBox_dbConfig},
+      pushButton_browseBackupDir {self.groupBox_dbConfig},
+      label_numBackups           {self.groupBox_dbConfig},
+      spinBox_numBackups         {self.groupBox_dbConfig},
+      label_frequency            {self.groupBox_dbConfig},
+      spinBox_frequency          {self.groupBox_dbConfig} {
       //
       // Optimise the select file dialog to select directories
       //
@@ -209,10 +184,20 @@ public:
       return;
    }
 
-   void initLangs(OptionDialog & optionDialog) {
+   void retranslateLanguageComboBox() {
+      QVector<Localization::LanguageInfo> const & languageInfos {Localization::languageInfo()};
+      for (int ii = 0; ii < languageInfos.size(); ++ii) {
+         QString nameInCurrentLang = tr(languageInfos[ii].nameInEnglish);
+         this->m_self.comboBox_lang->setItemText(ii, nameInCurrentLang);
+      }
+      return;
+   }
 
-      for (auto langInfo : this->languageInfo) {
-         optionDialog.comboBox_lang->addItem(langInfo.countryFlag, langInfo.nameInCurrentLang, langInfo.iso639_1Code);
+   void initLangs() {
+      QVector<Localization::LanguageInfo> const & languageInfos {Localization::languageInfo()};
+      for (auto langInfo : languageInfos) {
+         QString nameInCurrentLang = tr(langInfo.nameInEnglish);
+         this->m_self.comboBox_lang->addItem(langInfo.countryFlag, nameInCurrentLang, langInfo.iso639_1Code);
       }
 
       //
@@ -225,7 +210,7 @@ public:
       // RangedSlider.cpp.  But in the meantime, hopefully 36 wide × 24 high is an OK compromise for the various
       // different screen resolutions commonly in use.
       //
-      optionDialog.comboBox_lang->setIconSize(QSize(36, 24));
+      this->m_self.comboBox_lang->setIconSize(QSize(36, 24));
 
       return;
    }
@@ -235,7 +220,7 @@ public:
     */
    ~impl() = default;
 
-   void postgresVisible(bool canSee) {
+   void postgresVisible(bool const canSee) {
       this->label_pgHostname.setVisible(canSee);
       this->input_pgHostname.setVisible(canSee);
       this->label_pgPortNum.setVisible(canSee);
@@ -253,7 +238,7 @@ public:
       return;
    }
 
-   void sqliteVisible(bool canSee) {
+   void sqliteVisible(bool const canSee) {
       this->label_userDataDir.setVisible(canSee);
       this->input_userDataDir.setVisible(canSee);
       this->pushButton_browseDataDir.setVisible(canSee);
@@ -267,14 +252,13 @@ public:
       return;
    }
 
-
    /**
     *
     */
-   void clearLayout(OptionDialog & optionDialog) {
+   void clearLayout() {
       QLayoutItem * child;
-      while ((child = optionDialog.gridLayout->takeAt(0)) != nullptr) {
-         optionDialog.gridLayout->removeItem(child);
+      while ((child = this->m_self.gridLayout->takeAt(0)) != nullptr) {
+         this->m_self.gridLayout->removeItem(child);
       }
       return;
    }
@@ -282,59 +266,58 @@ public:
    /**
     * Determine which set of DB config params to show, based on whether PostgresSQL or SQLite is selected
     */
-   void setDbDialog(OptionDialog & optionDialog, Database::DbType db) {
+   void setDbDialog(Database::DbType db) {
       qDebug() <<
          Q_FUNC_INFO << "Set " << (db == Database::DbType::PGSQL ? "PostgresSQL" : "SQLite") << " config params visible";
-      optionDialog.groupBox_dbConfig->setVisible(false);
+      this->m_self.groupBox_dbConfig->setVisible(false);
 
-      this->clearLayout(optionDialog);
+      this->clearLayout();
       if (db == Database::DbType::PGSQL) {
          this->postgresVisible(true);
          this->sqliteVisible(false);
 
-         optionDialog.gridLayout->addWidget(&this->label_pgHostname, 0, 0);
-         optionDialog.gridLayout->addWidget(&this->input_pgHostname, 0, 1, 1, 2);
+         this->m_self.gridLayout->addWidget(&this->label_pgHostname, 0, 0);
+         this->m_self.gridLayout->addWidget(&this->input_pgHostname, 0, 1, 1, 2);
 
-         optionDialog.gridLayout->addWidget(&this->label_pgPortNum, 0, 3);
-         optionDialog.gridLayout->addWidget(&this->input_pgPortNum, 0, 4);
+         this->m_self.gridLayout->addWidget(&this->label_pgPortNum, 0, 3);
+         this->m_self.gridLayout->addWidget(&this->input_pgPortNum, 0, 4);
 
-         optionDialog.gridLayout->addWidget(&this->label_pgSchema, 1, 0);
-         optionDialog.gridLayout->addWidget(&this->input_pgSchema, 1, 1);
+         this->m_self.gridLayout->addWidget(&this->label_pgSchema, 1, 0);
+         this->m_self.gridLayout->addWidget(&this->input_pgSchema, 1, 1);
 
-         optionDialog.gridLayout->addWidget(&this->label_pgDbName, 2, 0);
-         optionDialog.gridLayout->addWidget(&this->input_pgDbName, 2, 1);
+         this->m_self.gridLayout->addWidget(&this->label_pgDbName, 2, 0);
+         this->m_self.gridLayout->addWidget(&this->input_pgDbName, 2, 1);
 
-         optionDialog.gridLayout->addWidget(&this->label_pgUsername, 3, 0);
-         optionDialog.gridLayout->addWidget(&this->input_pgUsername, 3, 1);
+         this->m_self.gridLayout->addWidget(&this->label_pgUsername, 3, 0);
+         this->m_self.gridLayout->addWidget(&this->input_pgUsername, 3, 1);
 
-         optionDialog.gridLayout->addWidget(&this->label_pgPassword, 4, 0);
-         optionDialog.gridLayout->addWidget(&this->input_pgPassword, 4, 1);
+         this->m_self.gridLayout->addWidget(&this->label_pgPassword, 4, 0);
+         this->m_self.gridLayout->addWidget(&this->input_pgPassword, 4, 1);
 
-         optionDialog.gridLayout->addWidget(&this->checkBox_savePgPassword, 4, 4);
+         this->m_self.gridLayout->addWidget(&this->checkBox_savePgPassword, 4, 4);
 
       } else {
          this->postgresVisible(false);
          this->sqliteVisible(true);
 
-         optionDialog.gridLayout->addWidget(&this->label_userDataDir, 0, 0);
-         optionDialog.gridLayout->addWidget(&this->input_userDataDir, 0, 1, 1, 2);
-         optionDialog.gridLayout->addWidget(&this->pushButton_browseDataDir, 0, 3);
+         this->m_self.gridLayout->addWidget(&this->label_userDataDir, 0, 0);
+         this->m_self.gridLayout->addWidget(&this->input_userDataDir, 0, 1, 1, 2);
+         this->m_self.gridLayout->addWidget(&this->pushButton_browseDataDir, 0, 3);
 
-         optionDialog.gridLayout->addWidget(&this->label_backupDir, 1, 0);
-         optionDialog.gridLayout->addWidget(&this->input_backupDir, 1, 1, 1, 2);
-         optionDialog.gridLayout->addWidget(&this->pushButton_browseBackupDir, 1, 3);
+         this->m_self.gridLayout->addWidget(&this->label_backupDir, 1, 0);
+         this->m_self.gridLayout->addWidget(&this->input_backupDir, 1, 1, 1, 2);
+         this->m_self.gridLayout->addWidget(&this->pushButton_browseBackupDir, 1, 3);
 
-         optionDialog.gridLayout->addWidget(&this->label_numBackups, 3, 0);
-         optionDialog.gridLayout->addWidget(&this->spinBox_numBackups, 3, 1);
+         this->m_self.gridLayout->addWidget(&this->label_numBackups, 3, 0);
+         this->m_self.gridLayout->addWidget(&this->spinBox_numBackups, 3, 1);
 
-         optionDialog.gridLayout->addWidget(&this->label_frequency, 4, 0);
-         optionDialog.gridLayout->addWidget(&this->spinBox_frequency, 4, 1);
+         this->m_self.gridLayout->addWidget(&this->label_frequency, 4, 0);
+         this->m_self.gridLayout->addWidget(&this->spinBox_frequency, 4, 1);
       }
-      optionDialog.groupBox_dbConfig->setVisible(true);
+      this->m_self.groupBox_dbConfig->setVisible(true);
       return;
    }
 
-
    void retranslateDbDialog() {
       //PostgreSQL stuff
       this->label_pgHostname.setText(QApplication::translate("optionsDialog", "Hostname", nullptr));
@@ -371,7 +354,7 @@ public:
                                                                 "Number of backups to keep: -1 means never remove, 0 means never backup", nullptr));
       // Actually the backups happen after every X times the program is closed, but the tooltip is already long enough!
       this->label_frequency.setToolTip(QApplication::translate("optionsDialog",
-                                                               "How many times Brewtarget needs to be run to trigger another backup: 1 means always backup", nullptr));
+                                                               "How many times %1 needs to be run to trigger another backup: 1 means always backup", nullptr).arg(CONFIG_APPLICATION_NAME_UC));
 #endif
       return;
    }
@@ -379,20 +362,17 @@ public:
    /**
     * \brief Update UI strings according to current language.
     */
-   void retranslate(OptionDialog & optionDialog) {
+   void retranslate() {
       // Let the Ui take care of its business
-      optionDialog.retranslateUi(&optionDialog);
+      this->m_self.retranslateUi(&this->m_self);
       this->retranslateDbDialog();
 
-      // Retranslate the language combobox.
-      for (int ii = 0; ii < this->languageInfo.size(); ++ii) {
-         this->languageInfo[ii].nameInCurrentLang = tr(this->languageInfo[ii].nameInEnglish);
-         optionDialog.comboBox_lang->setItemText(ii, this->languageInfo[ii].nameInCurrentLang);
-      }
+      this->retranslateLanguageComboBox();
+
       return;
    }
 
-   void changeColors(OptionDialog & optionDialog) {
+   void changeColors() {
       // Yellow when the test is needed
       // Red when the test failed
       // Green when the test passed
@@ -400,82 +380,84 @@ public:
 
       switch (dbConnectionTestState) {
          case NEEDS_TEST:
-            optionDialog.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
-            optionDialog.pushButton_testConnection->setEnabled(true);
-            optionDialog.pushButton_testConnection->setStyleSheet("color:rgb(240,225,25)");
+            this->m_self.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+            this->m_self.pushButton_testConnection->setEnabled(true);
+            this->m_self.pushButton_testConnection->setStyleSheet("color:rgb(240,225,25)");
             break;
          case TEST_FAILED:
-            optionDialog.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
-            optionDialog.pushButton_testConnection->setStyleSheet("color:red");
+            this->m_self.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
+            this->m_self.pushButton_testConnection->setStyleSheet("color:red");
             break;
          case TEST_PASSED:
-            optionDialog.pushButton_testConnection->setStyleSheet("color:green");
-            optionDialog.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
-            optionDialog.pushButton_testConnection->setEnabled(false);
+            this->m_self.pushButton_testConnection->setStyleSheet("color:green");
+            this->m_self.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
+            this->m_self.pushButton_testConnection->setEnabled(false);
             break;
          case NO_CHANGE:
-            optionDialog.pushButton_testConnection->setStyleSheet("color:grey");
-            optionDialog.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
-            optionDialog.pushButton_testConnection->setEnabled(false);
+            this->m_self.pushButton_testConnection->setStyleSheet("color:grey");
+            this->m_self.buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
+            this->m_self.pushButton_testConnection->setEnabled(false);
             break;
       }
       return;
    }
 
-   // Update dialog with current options.
-   void showChanges(OptionDialog & optionDialog) {
+   /**
+    * \brief Update dialog with current options.
+    */
+   void showChanges() {
       // Set the right language
-      int index = optionDialog.comboBox_lang->findData(Localization::getCurrentLanguage());
+      int index = this->m_self.comboBox_lang->findData(Localization::getCurrentLanguage());
       if (index >= 0) {
-         optionDialog.comboBox_lang->setCurrentIndex(index);
+         this->m_self.comboBox_lang->setCurrentIndex(index);
       }
 
-      optionDialog.weightComboBox->setCurrentIndex(
-         optionDialog.weightComboBox->findData(
+      this->m_self.weightComboBox->setCurrentIndex(
+         this->m_self.weightComboBox->findData(
             Measurement::getDisplayUnitSystem(Measurement::PhysicalQuantity::Mass).uniqueName
          )
       );
-      optionDialog.temperatureComboBox->setCurrentIndex(
-         optionDialog.temperatureComboBox->findData(
+      this->m_self.temperatureComboBox->setCurrentIndex(
+         this->m_self.temperatureComboBox->findData(
             Measurement::getDisplayUnitSystem(Measurement::PhysicalQuantity::Temperature).uniqueName
          )
       );
-      optionDialog.volumeComboBox->setCurrentIndex(
-         optionDialog.volumeComboBox->findData(
+      this->m_self.volumeComboBox->setCurrentIndex(
+         this->m_self.volumeComboBox->findData(
             Measurement::getDisplayUnitSystem(Measurement::PhysicalQuantity::Volume).uniqueName
          )
       );
-      optionDialog.gravityComboBox->setCurrentIndex(
-         optionDialog.gravityComboBox->findData(
+      this->m_self.gravityComboBox->setCurrentIndex(
+         this->m_self.gravityComboBox->findData(
             Measurement::getDisplayUnitSystem(Measurement::PhysicalQuantity::Density).uniqueName
          )
       );
-      optionDialog.dateComboBox->setCurrentIndex(optionDialog.dateComboBox->findData(Localization::getDateFormat()));
-      optionDialog.colorComboBox->setCurrentIndex(
-         optionDialog.colorComboBox->findData(
+      this->m_self.dateComboBox->setCurrentIndex(this->m_self.dateComboBox->findData(Localization::getDateFormat()));
+      this->m_self.colorComboBox->setCurrentIndex(
+         this->m_self.colorComboBox->findData(
             Measurement::getDisplayUnitSystem(Measurement::PhysicalQuantity::Color).uniqueName
          )
       );
-      optionDialog.diastaticPowerComboBox->setCurrentIndex(
-         optionDialog.diastaticPowerComboBox->findData(
+      this->m_self.diastaticPowerComboBox->setCurrentIndex(
+         this->m_self.diastaticPowerComboBox->findData(
             Measurement::getDisplayUnitSystem(Measurement::PhysicalQuantity::DiastaticPower).uniqueName
          )
       );
 
-      optionDialog.colorFormulaComboBox->setCurrentIndex(
-         optionDialog.colorFormulaComboBox->findData(ColorMethods::colorFormula)
+      this->m_self.colorFormulaComboBox->setCurrentIndex(
+         this->m_self.colorFormulaComboBox->findData(ColorMethods::colorFormula)
       );
-      optionDialog.ibuFormulaComboBox->setCurrentIndex(
-         optionDialog.ibuFormulaComboBox->findData(IbuMethods::ibuFormula)
+      this->m_self.ibuFormulaComboBox->setCurrentIndex(
+         this->m_self.ibuFormulaComboBox->findData(static_cast<int>(IbuMethods::ibuFormula))
       );
 
       // User data directory
-      this->input_userDataDir.setText(PersistentSettings::getUserDataDir().canonicalPath());
+      this->input_userDataDir.setText(PersistentSettings::getUserDataDir().absolutePath());
 
       // Backup stuff
       // By default backups go in the same directory as the DB
       this->input_backupDir.setText(PersistentSettings::value(PersistentSettings::Names::directory,
-                                                              PersistentSettings::getUserDataDir().canonicalPath(),
+                                                              PersistentSettings::getUserDataDir().absolutePath(),
                                                               PersistentSettings::Sections::backups).toString());
       this->spinBox_numBackups.setValue(PersistentSettings::value(PersistentSettings::Names::maximum,
                                                                   10,
@@ -489,56 +471,59 @@ public:
          PersistentSettings::value(PersistentSettings::Names::mashHopAdjustment, 0).toString(),
          Q_FUNC_INFO
       );
-      optionDialog.ibuAdjustmentMashHopDoubleSpinBox->setValue(amt * 100);
+      this->m_self.ibuAdjustmentMashHopDoubleSpinBox->setValue(amt * 100);
 
       amt = Localization::toDouble(
          PersistentSettings::value(PersistentSettings::Names::firstWortHopAdjustment, 1.1).toString(),
          Q_FUNC_INFO
       );
-      optionDialog.ibuAdjustmentFirstWortDoubleSpinBox->setValue(amt * 100);
+      this->m_self.ibuAdjustmentFirstWortDoubleSpinBox->setValue(amt * 100);
 
       // Database stuff -- this looks weird, but trust me. We want SQLITE to be
       // the default for this field
       int tmp = PersistentSettings::value(PersistentSettings::Names::dbType,
                                           static_cast<int>(Database::DbType::SQLITE)).toInt() - 1;
-      optionDialog.comboBox_engine->setCurrentIndex(tmp);
+      this->m_self.comboBox_engine->setCurrentIndex(tmp);
 
       this->input_pgHostname.setText(PersistentSettings::value(PersistentSettings::Names::dbHostname, "localhost").toString());
       this->input_pgPortNum.setText(PersistentSettings::value(PersistentSettings::Names::dbPortnum, "5432").toString());
       this->input_pgSchema.setText(PersistentSettings::value(PersistentSettings::Names::dbSchema, "public").toString());
-      this->input_pgDbName.setText(PersistentSettings::value(PersistentSettings::Names::dbName, "brewtarget").toString());
-      this->input_pgUsername.setText(PersistentSettings::value(PersistentSettings::Names::dbUsername, "brewtarget").toString());
+      this->input_pgDbName.setText(PersistentSettings::value(PersistentSettings::Names::dbName, CONFIG_APPLICATION_NAME_LC).toString());
+      this->input_pgUsername.setText(PersistentSettings::value(PersistentSettings::Names::dbUsername, CONFIG_APPLICATION_NAME_LC).toString());
       this->input_pgPassword.setText(PersistentSettings::value(PersistentSettings::Names::dbPassword, "").toString());
       this->checkBox_savePgPassword.setChecked(PersistentSettings::contains(PersistentSettings::Names::dbPassword));
 
       this->dbConnectionTestState = NO_CHANGE;
-      this->changeColors(optionDialog);
+      this->changeColors();
 
       if (RecipeHelper::getAutomaticVersioningEnabled()) {
-         optionDialog.checkBox_versioning->setCheckState(Qt::Checked);
-         optionDialog.groupBox_deleteBehavior->setEnabled(true);
+         this->m_self.checkBox_versioning->setCheckState(Qt::Checked);
+         this->m_self.groupBox_deleteBehavior->setEnabled(true);
          switch (PersistentSettings::value(PersistentSettings::Names::deletewhat, Recipe::DESCENDANT).toInt()) {
             case Recipe::ANCESTOR:
-               optionDialog.radioButton_deleteAncestor->setChecked(true);
+               this->m_self.radioButton_deleteAncestor->setChecked(true);
                break;
             default:
-               optionDialog.radioButton_deleteDescendant->setChecked(true);
+               this->m_self.radioButton_deleteDescendant->setChecked(true);
                break;
          }
       } else {
-         optionDialog.checkBox_versioning->setCheckState(Qt::Unchecked);
-         optionDialog.groupBox_deleteBehavior->setEnabled(false);
+         this->m_self.checkBox_versioning->setCheckState(Qt::Unchecked);
+         this->m_self.groupBox_deleteBehavior->setEnabled(false);
       }
 
       if (PersistentSettings::value(PersistentSettings::Names::showsnapshots, false).toBool()) {
-         optionDialog.checkBox_alwaysShowSnaps->setCheckState(Qt::Checked);
+         this->m_self.checkBox_alwaysShowSnaps->setCheckState(Qt::Checked);
       } else {
-         optionDialog.checkBox_alwaysShowSnaps->setCheckState(Qt::Unchecked);
+         this->m_self.checkBox_alwaysShowSnaps->setCheckState(Qt::Unchecked);
       }
 
       return;
    }
 
+   // =========================================== Member variables for impl ============================================
+   OptionDialog & m_self;
+
    // Used for selecting directories
    QFileDialog qFileDialog;
 
@@ -571,8 +556,6 @@ public:
 
    DbConnectionTestStates dbConnectionTestState;
 
-   QVector<LanguageInfo> languageInfo;
-
 };
 
 OptionDialog::OptionDialog(QWidget * parent) : QDialog{},
@@ -584,7 +567,7 @@ OptionDialog::OptionDialog(QWidget * parent) : QDialog{},
    // think it will end up biting my ...
    // anyway. It isn't pretty
    this->setupUi(this);
-   this->pimpl->initLangs(*this);
+   this->pimpl->initLangs();
 
    if (parent != nullptr) {
       setWindowIcon(parent->windowIcon());
@@ -606,7 +589,7 @@ OptionDialog::OptionDialog(QWidget * parent) : QDialog{},
    // figure out which database we have
    int idx = comboBox_engine->findData(PersistentSettings::value(PersistentSettings::Names::dbType,
                                                                  static_cast<int>(Database::DbType::SQLITE)).toInt());
-   this->pimpl->setDbDialog(*this, static_cast<Database::DbType>(idx));
+   this->pimpl->setDbDialog(static_cast<Database::DbType>(idx));
 
    // connect all the signals
    connect_signals();
@@ -652,13 +635,13 @@ void OptionDialog::configure_formulaCombos() {
                                    QVariant(Measurement::UnitSystems::diastaticPower_WindischKolbach.uniqueName));
 
    // Populate combo boxes on the "Formulas" tab
-   ibuFormulaComboBox->addItem(tr("Tinseth's approximation"), QVariant(IbuMethods::TINSETH));
-   ibuFormulaComboBox->addItem(tr("Rager's approximation"), QVariant(IbuMethods::RAGER));
-   ibuFormulaComboBox->addItem(tr("Noonan's approximation"), QVariant(IbuMethods::NOONAN));
+   ibuFormulaComboBox->addItem(tr("Tinseth's approximation"), QVariant(static_cast<int>(IbuMethods::IbuFormula::Tinseth)));
+   ibuFormulaComboBox->addItem(tr("Rager's approximation"  ), QVariant(static_cast<int>(IbuMethods::IbuFormula::Rager  )));
+   ibuFormulaComboBox->addItem(tr("Noonan's approximation" ), QVariant(static_cast<int>(IbuMethods::IbuFormula::Noonan )));
 
    colorFormulaComboBox->addItem(tr("Mosher's approximation"), QVariant(ColorMethods::MOSHER));
    colorFormulaComboBox->addItem(tr("Daniel's approximation"), QVariant(ColorMethods::DANIEL));
-   colorFormulaComboBox->addItem(tr("Morey's approximation"), QVariant(ColorMethods::MOREY));
+   colorFormulaComboBox->addItem(tr("Morey's approximation" ), QVariant(ColorMethods::MOREY));
 }
 
 void OptionDialog::configure_logging() {
@@ -685,7 +668,7 @@ void OptionDialog::connect_signals() {
    // figure out which database we have
    int idx = comboBox_engine->findData(PersistentSettings::value(PersistentSettings::Names::dbType,
                                                                  static_cast<int>(Database::DbType::SQLITE)).toInt());
-   this->pimpl->setDbDialog(*this, static_cast<Database::DbType>(idx));
+   this->pimpl->setDbDialog(static_cast<Database::DbType>(idx));
 
    // Set the signals
    connect(&this->pimpl->checkBox_savePgPassword, &QAbstractButton::clicked, this, &OptionDialog::savePassword);
@@ -708,7 +691,7 @@ void OptionDialog::connect_signals() {
    connect(checkBox_alwaysShowSnaps, &QAbstractButton::clicked, this, &OptionDialog::signalAncestors);
 
    // Call this here to set up translatable strings.
-   this->pimpl->retranslate(*this);
+   this->pimpl->retranslate();
    return;
 }
 
@@ -722,7 +705,7 @@ OptionDialog::~OptionDialog() = default;
 
 
 void OptionDialog::show() {
-   this->pimpl->showChanges(*this);
+   this->pimpl->showChanges();
    this->setVisible(true);
    return;
 }
@@ -760,13 +743,13 @@ void OptionDialog::resetToDefault() {
       this->pimpl->input_pgHostname.setText(QString("localhost"));
       this->pimpl->input_pgPortNum.setText(QString("5432"));
       this->pimpl->input_pgSchema.setText(QString("public"));
-      this->pimpl->input_pgDbName.setText(QString("brewtarget"));
-      this->pimpl->input_pgUsername.setText(QString("brewtarget"));
+      this->pimpl->input_pgDbName.setText(QString(CONFIG_APPLICATION_NAME_LC));
+      this->pimpl->input_pgUsername.setText(QString(CONFIG_APPLICATION_NAME_LC));
       this->pimpl->input_pgPassword.setText(QString(""));
       this->pimpl->checkBox_savePgPassword.setChecked(false);
    } else {
-      this->pimpl->input_userDataDir.setText(PersistentSettings::getConfigDir().canonicalPath());
-      this->pimpl->input_backupDir.setText(PersistentSettings::getConfigDir().canonicalPath());
+      this->pimpl->input_userDataDir.setText(PersistentSettings::getConfigDir().absolutePath());
+      this->pimpl->input_backupDir.setText(PersistentSettings::getConfigDir().absolutePath());
       this->pimpl->spinBox_frequency.setValue(4);
       this->pimpl->spinBox_numBackups.setValue(10);
    }
@@ -782,7 +765,7 @@ void OptionDialog::cancel() {
 void OptionDialog::changeEvent(QEvent * e) {
    switch (e->type()) {
       case QEvent::LanguageChange:
-         this->pimpl->retranslate(*this);
+         this->pimpl->retranslate();
          e->accept();
          break;
       default:
@@ -797,7 +780,7 @@ void OptionDialog::setEngine([[maybe_unused]] int selected) {
    QVariant data = comboBox_engine->currentData();
    Database::DbType newEngine = static_cast<Database::DbType>(data.toInt());
 
-   this->pimpl->setDbDialog(*this, newEngine);
+   this->pimpl->setDbDialog(newEngine);
    this->testRequired();
    return;
 }
@@ -838,13 +821,13 @@ void OptionDialog::testConnection() {
       // Database::testConnection already popped the dialog
       this->pimpl->dbConnectionTestState = TEST_FAILED;
    }
-   this->pimpl->changeColors(*this);
+   this->pimpl->changeColors();
    return;
 }
 
 void OptionDialog::testRequired() {
    this->pimpl->dbConnectionTestState = NEEDS_TEST;
-   this->pimpl->changeColors(*this);
+   this->pimpl->changeColors();
    return;
 }
 
@@ -902,7 +885,7 @@ void OptionDialog::saveFormulae() {
    bool okay = false;
 
    int ndx = ibuFormulaComboBox->itemData(ibuFormulaComboBox->currentIndex()).toInt(&okay);
-   IbuMethods::ibuFormula = static_cast<IbuMethods::IbuType>(ndx);
+   IbuMethods::ibuFormula = static_cast<IbuMethods::IbuFormula>(ndx);
    ndx = colorFormulaComboBox->itemData(colorFormulaComboBox->currentIndex()).toInt(&okay);
    ColorMethods::colorFormula = static_cast<ColorMethods::ColorType>(ndx);
 
@@ -948,10 +931,10 @@ bool OptionDialog::saveDatabaseConfig() {
    if (this->pimpl->dbConnectionTestState == NEEDS_TEST || this->pimpl->dbConnectionTestState == TEST_FAILED) {
       QMessageBox::critical(
          nullptr,
-                            tr("Test connection or cancel"),
-         tr("Saving the options without testing the connection can cause Brewtarget to not restart.  Your changes have "
+         tr("Test connection or cancel"),
+         tr("Saving the options without testing the connection can cause %1 to not restart.  Your changes have "
             "been discarded, which is likely really, really crappy UX.  Please open a bug explaining exactly how you "
-            "got to this message.")
+            "got to this message.").arg(CONFIG_APPLICATION_NAME_UC)
                            );
       return false;
    }
@@ -982,7 +965,8 @@ bool OptionDialog::transferDatabase() {
    // preserve the information required.
    try {
       QString theQuestion =
-         tr("Would you like Brewtarget to transfer your data to the new database? NOTE: If you've already loaded the data, say No");
+         tr("Would you like %1 to transfer your data to the new database? "
+            "NOTE: If you've already loaded the data, say No").arg(CONFIG_APPLICATION_NAME_UC);
       if (QMessageBox::Yes == QMessageBox::question(this, tr("Transfer database"), theQuestion)) {
          Database::instance().convertDatabase(this->pimpl->input_pgHostname.text(),
                                               this->pimpl->input_pgDbName.text(),
@@ -1002,7 +986,11 @@ bool OptionDialog::transferDatabase() {
          PersistentSettings::insert(PersistentSettings::Names::dbName,     this->pimpl->input_pgDbName.text());
          PersistentSettings::insert(PersistentSettings::Names::dbUsername, this->pimpl->input_pgUsername.text());
       }
-      QMessageBox::information(this, tr("Restart"), tr("Please restart Brewtarget to connect to the new database"));
+      QMessageBox::information(
+         this,
+         tr("Restart"),
+         tr("Please restart %1 to connect to the new database").arg(CONFIG_APPLICATION_NAME_UC)
+      );
    } catch (QString e) {
       qCritical() << Q_FUNC_INFO << e;
       success = false;
@@ -1032,7 +1020,7 @@ void OptionDialog::saveSqliteConfig() {
       QMessageBox::information(
          this,
          tr("Restart"),
-         tr("Please restart Brewtarget.")
+         tr("Please restart %1.").arg(CONFIG_APPLICATION_NAME_UC)
       );
    }
 
