@@ -1,23 +1,22 @@
-/*
- * WaterDialog.cpp is part of Brewtarget, and is Copyright the following
- * authors 2009-2023
- * - Matt Young <mfsy@yahoo.com>
- * - Maxime Lavigne <duguigne@gmail.com>
- * - Philip G. Lee <rocketman768@gmail.com>
+/*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
+ * WaterDialog.cpp is part of Brewtarget, and is copyright the following authors 2009-2024:
+ *   • Mattias Måhl <mattias@kejsarsten.com>
+ *   • Matt Young <mfsy@yahoo.com>
+ *   • Maxime Lavigne <duguigne@gmail.com>
+ *   • Mik Firestone <mikfire@gmail.com>
+ *   • Philip Greggory Lee <rocketman768@gmail.com>
  *
- * Brewtarget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+ * version.
  *
- * Brewtarget is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+ * You should have received a copy of the GNU General Public License along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌*/
 #include "WaterDialog.h"
 
 #include <limits>
@@ -27,21 +26,28 @@
 #include <QFont>
 #include <QInputDialog>
 
+#include "buttons/WaterButton.h"
 #include "database/ObjectStoreWrapper.h"
 #include "measurement/ColorMethods.h"
 #include "model/Fermentable.h"
 #include "model/Mash.h"
 #include "model/MashStep.h"
 #include "model/Recipe.h"
+#include "model/RecipeAdditionFermentable.h"
+#include "model/RecipeUseOfWater.h"
 #include "model/Salt.h"
-#include "tableModels/SaltTableModel.h"
-#include "tableModels/WaterTableModel.h"
-#include "WaterButton.h"
-#include "WaterEditor.h"
-#include "WaterListModel.h"
-#include "WaterSortFilterProxyModel.h"
+#include "qtModels/tableModels/RecipeAdjustmentSaltTableModel.h"
+#include "qtModels/tableModels/WaterTableModel.h"
+#include "editors/WaterEditor.h"
+#include "qtModels/listModels/WaterListModel.h"
+#include "qtModels/sortFilterProxyModels/WaterSortFilterProxyModel.h"
 #include "widgets/SmartDigitWidget.h"
 
+#ifdef BUILDING_WITH_CMAKE
+   // Explicitly doing this include reduces potential problems with AUTOMOC when compiling with CMake
+   #include "moc_WaterDialog.cpp"
+#endif
+
 
 //
 // All of the pH calculations are taken from the work done by Kai Troester and published at
@@ -70,8 +76,8 @@ namespace {
 
 WaterDialog::WaterDialog(QWidget* parent) :
    QDialog{parent},
-   m_ppm_digits{  QVector<SmartDigitWidget*>{static_cast<int>(Water::Ions::numIons)} },
-   m_total_digits{QVector<SmartDigitWidget*>{static_cast<int>(Salt::Types::numTypes)} },
+   m_ppm_digits  {QVector<SmartDigitWidget*>{Water::ionStringMapping.size()} },
+   m_total_digits{QVector<SmartDigitWidget*>{Salt::typeStringMapping.size()} },
    m_rec{nullptr},
    m_base{nullptr},
    m_target{nullptr},
@@ -98,13 +104,13 @@ WaterDialog::WaterDialog(QWidget* parent) :
    m_target_filter->sort(0);
    targetProfileCombo->setModel(m_target_filter);
 
-   SMART_FIELD_INIT_FS(WaterDialog, label_ca  , btDigit_ca  , double, Measurement::PhysicalQuantity::VolumeConcentration, 2);
-   SMART_FIELD_INIT_FS(WaterDialog, label_cl  , btDigit_cl  , double, Measurement::PhysicalQuantity::VolumeConcentration, 2);
-   SMART_FIELD_INIT_FS(WaterDialog, label_hco3, btDigit_hco3, double, Measurement::PhysicalQuantity::VolumeConcentration, 2);
-   SMART_FIELD_INIT_FS(WaterDialog, label_mg  , btDigit_mg  , double, Measurement::PhysicalQuantity::VolumeConcentration, 2);
-   SMART_FIELD_INIT_FS(WaterDialog, label_na  , btDigit_na  , double, Measurement::PhysicalQuantity::VolumeConcentration, 2);
-   SMART_FIELD_INIT_FS(WaterDialog, label_so4 , btDigit_so4 , double, Measurement::PhysicalQuantity::VolumeConcentration, 2);
-   SMART_FIELD_INIT_FS(WaterDialog, label_pH  , btDigit_ph  , double, Measurement::PhysicalQuantity::Acidity            , 1);
+   SMART_FIELD_INIT_FS(WaterDialog, label_ca  , btDigit_ca  , double, Measurement::PhysicalQuantity::MassFractionOrConc, 2);
+   SMART_FIELD_INIT_FS(WaterDialog, label_cl  , btDigit_cl  , double, Measurement::PhysicalQuantity::MassFractionOrConc, 2);
+   SMART_FIELD_INIT_FS(WaterDialog, label_hco3, btDigit_hco3, double, Measurement::PhysicalQuantity::MassFractionOrConc, 2);
+   SMART_FIELD_INIT_FS(WaterDialog, label_mg  , btDigit_mg  , double, Measurement::PhysicalQuantity::MassFractionOrConc, 2);
+   SMART_FIELD_INIT_FS(WaterDialog, label_na  , btDigit_na  , double, Measurement::PhysicalQuantity::MassFractionOrConc, 2);
+   SMART_FIELD_INIT_FS(WaterDialog, label_so4 , btDigit_so4 , double, Measurement::PhysicalQuantity::MassFractionOrConc, 2);
+   SMART_FIELD_INIT_FS(WaterDialog, label_pH  , btDigit_ph  , double, Measurement::PhysicalQuantity::Acidity           , 1);
 
    SMART_FIELD_INIT_FS(WaterDialog, label_totalcacl2 , btDigit_totalcacl2 , double, Measurement::PhysicalQuantity::Mass, 2);
    SMART_FIELD_INIT_FS(WaterDialog, label_totalcaco3 , btDigit_totalcaco3 , double, Measurement::PhysicalQuantity::Mass, 2);
@@ -114,43 +120,42 @@ WaterDialog::WaterDialog(QWidget* parent) :
    SMART_FIELD_INIT_FS(WaterDialog, label_totalnahco3, btDigit_totalnahco3, double, Measurement::PhysicalQuantity::Mass, 2);
 
    // not sure if this is better or worse, but we will try it out
-   m_ppm_digits[static_cast<int>(Water::Ions::Ca)]   = btDigit_ca  ;
-   m_ppm_digits[static_cast<int>(Water::Ions::Cl)]   = btDigit_cl  ;
-   m_ppm_digits[static_cast<int>(Water::Ions::HCO3)] = btDigit_hco3;
-   m_ppm_digits[static_cast<int>(Water::Ions::Mg)]   = btDigit_mg  ;
-   m_ppm_digits[static_cast<int>(Water::Ions::Na)]   = btDigit_na  ;
-   m_ppm_digits[static_cast<int>(Water::Ions::SO4)]  = btDigit_so4 ;
+   m_ppm_digits[static_cast<int>(Water::Ion::Ca)]   = btDigit_ca  ;
+   m_ppm_digits[static_cast<int>(Water::Ion::Cl)]   = btDigit_cl  ;
+   m_ppm_digits[static_cast<int>(Water::Ion::HCO3)] = btDigit_hco3;
+   m_ppm_digits[static_cast<int>(Water::Ion::Mg)]   = btDigit_mg  ;
+   m_ppm_digits[static_cast<int>(Water::Ion::Na)]   = btDigit_na  ;
+   m_ppm_digits[static_cast<int>(Water::Ion::SO4)]  = btDigit_so4 ;
 
-   m_total_digits[static_cast<int>(Salt::Types::CACL2 )] = btDigit_totalcacl2 ;
-   m_total_digits[static_cast<int>(Salt::Types::CACO3 )] = btDigit_totalcaco3 ;
-   m_total_digits[static_cast<int>(Salt::Types::CASO4 )] = btDigit_totalcaso4 ;
-   m_total_digits[static_cast<int>(Salt::Types::MGSO4 )] = btDigit_totalmgso4 ;
-   m_total_digits[static_cast<int>(Salt::Types::NACL  )] = btDigit_totalnacl  ;
-   m_total_digits[static_cast<int>(Salt::Types::NAHCO3)] = btDigit_totalnahco3;
+   m_total_digits[static_cast<int>(Salt::Type::CaCl2 )] = btDigit_totalcacl2 ;
+   m_total_digits[static_cast<int>(Salt::Type::CaCO3 )] = btDigit_totalcaco3 ;
+   m_total_digits[static_cast<int>(Salt::Type::CaSO4 )] = btDigit_totalcaso4 ;
+   m_total_digits[static_cast<int>(Salt::Type::MgSO4 )] = btDigit_totalmgso4 ;
+   m_total_digits[static_cast<int>(Salt::Type::NaCl  )] = btDigit_totalnacl  ;
+   m_total_digits[static_cast<int>(Salt::Type::NaHCO3)] = btDigit_totalnahco3;
 
-   // foreach( SmartDigitWidget* i, m_ppm_digits ) {
-   for (int i = 0; i < static_cast<int>(Water::Ions::numIons); ++i ) {
-      m_ppm_digits[i]->setLimits(0.0,1000.0);
-      m_ppm_digits[i]->setAmount(0.0);
-      m_ppm_digits[i]->setMessages(tr("Too low for target profile."),
-                                   tr("In range for target profile."),
-                                   tr("Too high for target profile."));
+   for (int ii = 0; ii < Water::ionStringMapping.size(); ++ii) {
+      m_ppm_digits[ii]->setLimits(0.0,1000.0);
+      m_ppm_digits[ii]->setQuantity(0.0);
+      m_ppm_digits[ii]->setMessages(tr("Too low for target profile."),
+                                    tr("In range for target profile."),
+                                    tr("Too high for target profile."));
    }
    // we can be a bit more specific with pH
    btDigit_ph->setLowLim(5.0);
    btDigit_ph->setHighLim(5.5);
-   btDigit_ph->setAmount(7.0);
+   btDigit_ph->setQuantity(7.0);
 
    // since all the things are now digits, lets get the totals configured
-   for (int i = static_cast<int>(Salt::Types::CACL2); i < static_cast<int>(Salt::Types::NAHCO3); ++i ) {
-      m_total_digits[i]->setConstantColor(SmartDigitWidget::ColorType::Black);
-      m_total_digits[i]->setAmount(0.0);
+   for (int ii = static_cast<int>(Salt::Type::CaCl2); ii < static_cast<int>(Salt::Type::NaHCO3); ++ii ) {
+      m_total_digits[ii]->setConstantColor(SmartDigitWidget::ColorType::Black);
+      m_total_digits[ii]->setQuantity(0.0);
    }
    // and now let's see what the table does.
-   m_salt_table_model = new SaltTableModel(tableView_salts);
-   m_salt_delegate    = new SaltItemDelegate(tableView_salts);
-   tableView_salts->setItemDelegate(m_salt_delegate);
-   tableView_salts->setModel(m_salt_table_model);
+   m_saltAdjustmentTableModel = new RecipeAdjustmentSaltTableModel(tableView_saltAdjustments);
+   m_saltAdjustmentDelegate   = new RecipeAdjustmentSaltItemDelegate(tableView_saltAdjustments, *m_saltAdjustmentTableModel);
+   tableView_saltAdjustments->setItemDelegate(m_saltAdjustmentDelegate);
+   tableView_saltAdjustments->setModel(m_saltAdjustmentTableModel);
 
    m_base_editor   = new WaterEditor(this, "Base");
    m_target_editor = new WaterEditor(this, "Target");
@@ -162,9 +167,9 @@ WaterDialog::WaterDialog(QWidget* parent) :
    connect(baseProfileButton,   &WaterButton::clicked, m_base_editor,   &QWidget::show);
    connect(targetProfileButton, &WaterButton::clicked, m_target_editor, &QWidget::show);
 
-   connect(m_salt_table_model,    &SaltTableModel::newTotals, this,               &WaterDialog::newTotals   );
-   connect(pushButton_addSalt,    &QAbstractButton::clicked,  m_salt_table_model, &SaltTableModel::catchSalt);
-   connect(pushButton_removeSalt, &QAbstractButton::clicked,  this,               &WaterDialog::removeSalts );
+   connect(m_saltAdjustmentTableModel, &RecipeAdjustmentSaltTableModel::newTotals, this                      , &WaterDialog::newTotals   );
+   connect(pushButton_addSalt        , &QAbstractButton::clicked                 , m_saltAdjustmentTableModel, &RecipeAdjustmentSaltTableModel::catchSalt);
+   connect(pushButton_removeSalt     , &QAbstractButton::clicked                 , this                      , &WaterDialog::removeSalts );
 
    connect(spinBox_mashRO,   QOverload<int>::of(&QSpinBox::valueChanged), this, &WaterDialog::setMashRO  );
    connect(spinBox_spargeRO, QOverload<int>::of(&QSpinBox::valueChanged), this, &WaterDialog::setSpargeRO);
@@ -179,14 +184,14 @@ WaterDialog::~WaterDialog() = default;
 
 void WaterDialog::setMashRO(int val) {
    m_mashRO = val/100.0;
-   if ( m_base ) m_base->setMashRO(m_mashRO);
+   if ( m_base ) m_base->setMashRo_pct(m_mashRO);
    newTotals();
    return;
 }
 
 void WaterDialog::setSpargeRO(int val) {
    m_spargeRO = val/100.0;
-   if ( m_base ) m_base->setSpargeRO(m_spargeRO);
+   if ( m_base ) m_base->setSpargeRo_pct(m_spargeRO);
    newTotals();
    return;
 }
@@ -196,12 +201,12 @@ void WaterDialog::setDigits() {
       return;
    }
 
-   for (int i = 0; i < static_cast<int>(Water::Ions::numIons); ++i ) {
-      double ppm = this->m_target->ppm(static_cast<Water::Ions>(i));
+   for (int ii = 0; ii < Water::ionStringMapping.size(); ++ii) {
+      double ppm = this->m_target->ppm(static_cast<Water::Ion>(ii));
       double min_ppm = ppm * 0.95;
       double max_ppm = ppm * 1.05;
-      m_ppm_digits[i]->setLimits(min_ppm,max_ppm);
-      m_ppm_digits[i]->setMessages(tr("Minimum expected concentration is %1 ppm").arg(min_ppm),
+      m_ppm_digits[ii]->setLimits(min_ppm,max_ppm);
+      m_ppm_digits[ii]->setMessages(tr("Minimum expected concentration is %1 ppm").arg(min_ppm),
                                    tr("In range for target profile."),
                                    tr("Maximum expected concentration is %1 ppm").arg(max_ppm));
    }
@@ -215,55 +220,90 @@ void WaterDialog::setRecipe(Recipe *rec) {
       return;
    }
 
-   m_rec = rec;
-   Mash* mash = m_rec->mash();
-   m_salt_table_model->observeRecipe(m_rec);
-   m_salt_delegate->observeRecipe(m_rec);
+   this->m_rec = rec;
+   auto mash = this->m_rec->mash();
+   m_saltAdjustmentTableModel->observeRecipe(this->m_rec);
 
-   if ( mash == nullptr || mash->mashSteps().size() == 0 ) {
+   if (!mash || mash->mashSteps().size() == 0 ) {
       qWarning() << QString("Cannot set water chemistry without a mash");
       return;
    }
 
-   baseProfileButton->setRecipe(m_rec);
-   targetProfileButton->setRecipe(m_rec);
+   baseProfileButton->setRecipe(this->m_rec);
+   targetProfileButton->setRecipe(this->m_rec);
 
-   for (auto waterId : this->m_rec->getWaterIds()) {
-      auto water = ObjectStoreWrapper::getById<Water>(waterId);
-      if (water->type() == Water::Types::BASE) {
+   for (auto waterUse : this->m_rec->waterUses()) {
+      auto water = ObjectStoreWrapper::getSharedFromRaw(waterUse->water());
+      if (water->type() == Water::Type::Base) {
          qDebug() << Q_FUNC_INFO << "Base Water" << *water;
          this->m_base = water;
-      } else if (water->type() == Water::Types::TARGET) {
+      } else if (water->type() == Water::Type::Target) {
          qDebug() << Q_FUNC_INFO << "Target Water" << *water;
          this->m_target = water;
       }
    }
 
    // I need these numbers before we set the ranges
-   for (Fermentable *i : m_rec->fermentables() ) {
-      m_total_grains += i->amount_kg();
+   for (auto const & fermentableAddition : m_rec->fermentableAdditions() ) {
+      // .:TBD:. This almost certainly needs some refinement
+      switch (fermentableAddition->fermentable()->type()) {
+         case Fermentable::Type::Grain:
+         case Fermentable::Type::Extract:
+         case Fermentable::Type::Dry_Extract:
+            if (fermentableAddition->getMeasure() == Measurement::PhysicalQuantity::Mass) {
+               m_total_grains += fermentableAddition->amount().quantity;
+            }
+            break;
+         case Fermentable::Type::Sugar:
+         case Fermentable::Type::Other_Adjunct:
+         case Fermentable::Type::Fruit:
+         case Fermentable::Type::Juice:
+         case Fermentable::Type::Honey:
+            // For the moment, at least, assume these types of fermentables do not affect color.  .:TBD:. This is
+            // probably wrong!
+            break;
+      }
    }
 
-   for (Fermentable *i : m_rec->fermentables() ) {
-      double lovi = ( i->color_srm() +0.6 ) / 1.35;
-      m_weighted_colors   += (i->amount_kg()/m_total_grains)*lovi;
+   // Now we've got m_total_grains, we need to loop over fermentable again
+   for (auto const & fermentableAddition : m_rec->fermentableAdditions() ) {
+      // .:TBD:. This almost certainly needs some refinement
+      switch (fermentableAddition->fermentable()->type()) {
+         case Fermentable::Type::Grain:
+         case Fermentable::Type::Extract:
+         case Fermentable::Type::Dry_Extract:
+            if (fermentableAddition->getMeasure() == Measurement::PhysicalQuantity::Mass) {
+               double lovi = (fermentableAddition->fermentable()->color_srm() +0.6 ) / 1.35;
+               m_weighted_colors   += (fermentableAddition->amount().quantity/m_total_grains)*lovi;
+            }
+            break;
+         case Fermentable::Type::Sugar:
+         case Fermentable::Type::Other_Adjunct:
+         case Fermentable::Type::Fruit:
+         case Fermentable::Type::Juice:
+         case Fermentable::Type::Honey:
+            // For the moment, at least, assume these types of fermentables do not affect color.  .:TBD:. This is
+            // probably wrong!
+            break;
+      }
    }
+
    m_thickness = m_rec->mash()->totalInfusionAmount_l()/m_total_grains;
 
    if (this->m_base) {
 
-      m_mashRO = this->m_base->mashRO();
-      spinBox_mashRO->setValue( QVariant(m_mashRO*100).toInt());
-      m_spargeRO = this->m_base->spargeRO();
-      spinBox_spargeRO->setValue( QVariant(m_spargeRO*100).toInt());
+      m_mashRO = this->m_base->mashRo_pct().value_or(0.0);
+      spinBox_mashRO->setValue( QVariant(m_mashRO * 100).toInt());
+      m_spargeRO = this->m_base->spargeRo_pct().value_or(0.0);
+      spinBox_spargeRO->setValue( QVariant(m_spargeRO * 100).toInt());
 
-      baseProfileButton->setWater(this->m_base.get());
-      m_base_editor->setWater(this->m_base);
+      baseProfileButton->setWater(this->m_base);
+      m_base_editor->setEditItem(this->m_base);
       // all of the magic to set the sliders happens in newTotals(). So don't do it twice
    }
    if (this->m_target && this->m_target != this->m_base) {
-      targetProfileButton->setWater(this->m_target.get());
-      m_target_editor->setWater(this->m_target);
+      targetProfileButton->setWater(this->m_target);
+      m_target_editor->setEditItem(this->m_target);
 
       this->setDigits();
    }
@@ -287,11 +327,11 @@ void WaterDialog::update_baseProfile(int selected) {
       // however that we do need to ensure the link to the "parent" water is not lost - hence the call to makeChild().
       this->m_base = std::make_shared<Water>(*parent);
       this->m_base->makeChild(*parent);
-      this->m_base->setType(Water::Types::BASE);
+      this->m_base->setType(Water::Type::Base);
       qDebug() << Q_FUNC_INFO << "Made base child" << *this->m_base << "from parent" << parent;
 
-      baseProfileButton->setWater(this->m_base.get());
-      m_base_editor->setWater(this->m_base);
+      baseProfileButton->setWater(this->m_base);
+      m_base_editor->setEditItem(this->m_base);
       newTotals();
    }
    return;
@@ -311,11 +351,11 @@ void WaterDialog::update_targetProfile(int selected) {
       // Comment above for copy of m_base applies equally here
       this->m_target = std::make_shared<Water>(*parent);
       this->m_target->makeChild(*parent);
-      this->m_target->setType(Water::Types::TARGET);
+      this->m_target->setType(Water::Type::Target);
       qDebug() << Q_FUNC_INFO << "Made target child" << *this->m_target << "from parent" << parent;
 
-      targetProfileButton->setWater(this->m_target.get());
-      m_target_editor->setWater(this->m_target);
+      targetProfileButton->setWater(this->m_target);
+      m_target_editor->setEditItem(this->m_target);
 
       this->setDigits();
    }
@@ -323,10 +363,11 @@ void WaterDialog::update_targetProfile(int selected) {
 }
 
 void WaterDialog::newTotals() {
-   if ( ! m_rec || ! m_rec->mash() )
+   if (!this->m_rec || !this->m_rec->mash()) {
       return;
+   }
 
-   Mash* mash = m_rec->mash();
+   auto mash = m_rec->mash();
    double allTheWaters = mash->totalMashWater_l();
 
    if ( qFuzzyCompare(allTheWaters,0.0) ) {
@@ -337,9 +378,10 @@ void WaterDialog::newTotals() {
    //   o the totals need to be updated
    //   o the digits need to be updated
 
-   for (int i = static_cast<int>(Salt::Types::CACL2); i < static_cast<int>(Salt::Types::LACTIC); ++i ) {
-      Salt::Types type = static_cast<Salt::Types>(i);
-      m_total_digits[i]->setAmount(m_salt_table_model->total(type));
+   // .:TBD:. It seems the ordering of Salt::Type is important.  Would be good to decouple this!
+   for (int ii = static_cast<int>(Salt::Type::CaCl2); ii < static_cast<int>(Salt::Type::LacticAcid); ++ii ) {
+      Salt::Type type = static_cast<Salt::Type>(ii);
+      m_total_digits[ii]->setQuantity(m_saltAdjustmentTableModel->total(type));
    }
 
    // the total_* method return the numerator, we supply the denominator and
@@ -354,31 +396,29 @@ void WaterDialog::newTotals() {
       // I hope this is right. All this 'rithmetic is making me head hurt.
       double modifier = 1.0 - (dInfuse + dSparge) / allTheWaters;
 
-      for (int i = 0; i < static_cast<int>(Water::Ions::numIons); ++i ) {
-         Water::Ions ion = static_cast<Water::Ions>(i);
+      for (int ii = 0; ii < Water::ionStringMapping.size(); ++ii) {
+         Water::Ion ion = static_cast<Water::Ion>(ii);
          double mPPM = modifier * this->m_base->ppm(ion);
-         m_ppm_digits[i]->setAmount( m_salt_table_model->total(ion) / allTheWaters + mPPM);
+         m_ppm_digits[ii]->setQuantity( m_saltAdjustmentTableModel->total(ion) / allTheWaters + mPPM);
 
       }
-      btDigit_ph->setAmount(calculateMashpH());
+      btDigit_ph->setQuantity(calculateMashpH());
 
    } else {
-      for (int i = 0; i < static_cast<int>(Water::Ions::numIons); ++i ) {
-         Water::Ions ion = static_cast<Water::Ions>(i);
-         m_ppm_digits[i]->setAmount( m_salt_table_model->total(ion) / allTheWaters);
+      for (int ii = 0; ii < Water::ionStringMapping.size(); ++ii) {
+         Water::Ion ion = static_cast<Water::Ion>(ii);
+         m_ppm_digits[ii]->setQuantity( m_saltAdjustmentTableModel->total(ion) / allTheWaters);
       }
    }
    return;
 }
 
 void WaterDialog::removeSalts() {
-   QModelIndexList selected = tableView_salts->selectionModel()->selectedIndexes();
-   QList<int> deadSalts;
+   QModelIndexList selected = tableView_saltAdjustments->selectionModel()->selectedIndexes();
 
-   for (QModelIndex i : selected) {
-      deadSalts.append( i.row() );
+   for (QModelIndex ii : selected) {
+      m_saltAdjustmentTableModel->remove(ii);
    }
-   m_salt_table_model->removeSalts(deadSalts);
    return;
 }
 
@@ -387,7 +427,7 @@ double WaterDialog::calculateRA() const {
    double residual = 0.0;
    if (this->m_base) {
 
-      double base_alk = ( 1.0 - m_base->mashRO() ) * m_base->alkalinity();
+      double base_alk = ( 1.0 - m_base->mashRo_pct().value_or(0.0) ) * m_base->alkalinity_ppm().value_or(0.0);
       if (!m_base->alkalinityAsHCO3()) {
          base_alk = 1.22 * base_alk;
       }
@@ -405,7 +445,7 @@ double WaterDialog::calculateSaltpH() {
       return 0.0;
    }
 
-   Mash* mash = m_rec->mash();
+   auto mash = m_rec->mash();
    double allTheWaters = mash->totalMashWater_l();
 
    double modifier = 1 - ( (m_mashRO * mash->totalInfusionAmount_l()) + (m_spargeRO * mash->totalSpargeAmount_l())) / allTheWaters;
@@ -430,10 +470,10 @@ double WaterDialog::calculateAddedSaltpH() {
 
    // We need the value from the salt table model, because we need all the
    // added salts, but not the base.
-   double ca   = this->m_salt_table_model->total_Ca()/Cagpm * 2;
-   double mg   = this->m_salt_table_model->total_Mg()/Mggpm * 2;
-   double hco3 = this->m_salt_table_model->total_HCO3()/HCO3gpm;
-   double co3  = this->m_salt_table_model->total_CO3()/CO3gpm;
+   double ca   = this->m_saltAdjustmentTableModel->total_Ca()/Cagpm * 2;
+   double mg   = this->m_saltAdjustmentTableModel->total_Mg()/Mggpm * 2;
+   double hco3 = this->m_saltAdjustmentTableModel->total_HCO3()/HCO3gpm;
+   double co3  = this->m_saltAdjustmentTableModel->total_CO3()/CO3gpm;
 
    // The 61 is another magic number from Kai. Sigh
    // unlike previous calculations, I am getting a mass here so I do not
@@ -449,9 +489,9 @@ double WaterDialog::calculateAcidpH() {
    const double lactic_gpm = 90;
    double totalDelta = 0.0;
 
-   double lactic_amt   = this->m_salt_table_model->totalAcidWeight(Salt::Types::LACTIC);
-   double acidmalt_amt = this->m_salt_table_model->totalAcidWeight(Salt::Types::ACIDMLT);
-   double H3PO4_amt    = this->m_salt_table_model->totalAcidWeight(Salt::Types::H3PO4);
+   double lactic_amt   = this->m_saltAdjustmentTableModel->totalAcidWeight(Salt::Type::LacticAcid);
+   double acidmalt_amt = this->m_saltAdjustmentTableModel->totalAcidWeight(Salt::Type::AcidulatedMalt);
+   double H3PO4_amt    = this->m_saltAdjustmentTableModel->totalAcidWeight(Salt::Type::H3PO4);
 
    if ( lactic_amt + acidmalt_amt > 0.0 ) {
       totalDelta += 1000 * (lactic_amt + acidmalt_amt) / lactic_gpm;
@@ -469,23 +509,37 @@ double WaterDialog::calculateGristpH() {
    double gristPh = nosrmbeer_ph;
    double pHAdjustment = 0.0;
 
-   if ( m_rec && m_rec->fermentables().size() ) {
+   if ( m_rec && m_rec->fermentableAdditions().size() ) {
 
       double platoRatio = 1/Measurement::Units::plato.fromCanonical(m_rec->og());
       double color = m_rec->color_srm();
       double colorFromGrain = 0.0;
 
-      for (Fermentable * i : m_rec->fermentables()) {
+      for (auto const & fermentableAddition : m_rec->fermentableAdditions() ) {
+         switch (fermentableAddition->fermentable()->type()) {
+            case Fermentable::Type::Grain:
+            case Fermentable::Type::Extract:
+            case Fermentable::Type::Dry_Extract:
                // I am counting anything that doesn't have diastatic
                // power as a roasted/crystal malt. I am sure my assumption will
                // haunt me later, but I have no way of knowing what kind of malt
                // (base, crystal, roasted) this is.
-         if ( i->diastaticPower_lintner() < 1 ) {
+               if (fermentableAddition->fermentable()->diastaticPower_lintner() < 1 ) {
                   double lovi = 19.0;
-            if ( i->color_srm() <= 120 ) {
-               lovi = ( i->color_srm() + 0.6)/1.35;
+                  if (fermentableAddition->fermentable()->color_srm() <= 120 ) {
+                     lovi = (fermentableAddition->fermentable()->color_srm() + 0.6)/1.35;
                }
-            colorFromGrain = ( i->amount_kg() / m_total_grains ) * lovi;
+                  colorFromGrain = (fermentableAddition->amount().quantity / m_total_grains ) * lovi;
+               }
+               break;
+            case Fermentable::Type::Sugar:
+            case Fermentable::Type::Other_Adjunct:
+            case Fermentable::Type::Fruit:
+            case Fermentable::Type::Juice:
+            case Fermentable::Type::Honey:
+               // For the moment, at least, assume these types of fermentables do not affect color.  .:TBD:. This is
+               // probably wrong!
+               break;
          }
       }
       double colorRatio = colorFromGrain/m_weighted_colors;
@@ -502,7 +556,7 @@ double WaterDialog::calculateGristpH() {
 double WaterDialog::calculateMashpH() {
    double mashpH = 0.0;
 
-   if ( m_rec && m_rec->fermentables().size() ) {
+   if ( m_rec && m_rec->fermentableAdditions().size() ) {
       double gristpH   = calculateGristpH();
       double basepH    = calculateSaltpH();
       double saltpH    = calculateAddedSaltpH();
@@ -517,14 +571,20 @@ double WaterDialog::calculateMashpH() {
 }
 
 void WaterDialog::saveAndClose() {
-   this->m_salt_table_model->saveAndClose();
+   this->m_saltAdjustmentTableModel->saveAndClose();
    if (this->m_base && this->m_base->key() < 0) {
       // Recipe will take care of adding to the relevant object store
-      this->m_rec->add(this->m_base);
+      auto waterUse = std::make_shared<RecipeUseOfWater>(tr("Use of %1").arg(this->m_base->name()),
+                                                         this->m_rec->key(),
+                                                         this->m_base->key());
+      this->m_rec->addAddition(waterUse);
    }
    if (this->m_target && this->m_target->key() < 0) {
       // Recipe will take care of adding to the relevant object store
-      this->m_rec->add(this->m_target);
+      auto waterUse = std::make_shared<RecipeUseOfWater>(tr("Use of %1").arg(this->m_target->name()),
+                                                         this->m_rec->key(),
+                                                         this->m_target->key());
+      this->m_rec->addAddition(waterUse);
    }
 
    setVisible(false);
