@@ -1,21 +1,18 @@
-/*
- * database/ObjectStoreTyped.h is part of Brewtarget, and is copyright the
- * following authors 2021-2023:
+/*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
+ * database/ObjectStoreTyped.h is part of Brewtarget, and is copyright the following authors 2021-2024:
  *   • Matt Young <mfsy@yahoo.com>
  *
- * Brewtarget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+ * version.
  *
- * Brewtarget is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+ * You should have received a copy of the GNU General Public License along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌*/
 #ifndef DATABASE_OBJECTSTORETYPED_H
 #define DATABASE_OBJECTSTORETYPED_H
 #pragma once
@@ -42,7 +39,7 @@ public:
    ObjectStoreTyped(TypeLookup               const & typeLookup,
                     TableDefinition          const & primaryTable,
                     JunctionTableDefinitions const & junctionTables = JunctionTableDefinitions{}) :
-      ObjectStore(typeLookup, primaryTable, junctionTables) {
+      ObjectStore(NE::staticMetaObject.className(), typeLookup, primaryTable, junctionTables) {
       return;
    }
 
@@ -52,8 +49,11 @@ public:
 
    /**
     * \brief Get the singleton instance of this class
+    *
+    * \param database Sets and stores the Database this store is going to work with.  Normally should only set when
+    *                 called from \c CreateAllDatabaseTables.
     */
-   static ObjectStoreTyped<NE> & getInstance();
+   static ObjectStoreTyped<NE> & getInstance(Database * database = nullptr);
 
    using ObjectStore::insert;
 
@@ -146,6 +146,10 @@ public:
     */
    std::shared_ptr<NE> getById(int id) const {
       if (!this->contains(id)) {
+         qDebug() << Q_FUNC_INFO << "ID" << id << "not found amongst" << this->size() << "objects";
+         // Uncomment the following to track down errors where we're looking for an object that is not in the DB.
+         // (Often this is because we are searching for an object before it has been stored in the DB.)
+//         qDebug().noquote() << Q_FUNC_INFO << Logging::getStackTrace();
          return nullptr;
       }
       return std::static_pointer_cast<NE>(this->ObjectStore::getById(id));
@@ -195,10 +199,10 @@ public:
     *
     * \param matchFunction Takes a pointer to an object and returns \c true if the object is a match or \c false otherwise.
     *
-    * \return Shared pointer to the first object that gives a \c true result to \c matchFunction, or \c std::nullopt if
-    *         none does
+    * \return Shared pointer to the first object that gives a \c true result to \c matchFunction, or \c nullptr if none
+    *         does.
     */
-   std::optional< std::shared_ptr<NE> > findFirstMatching(std::function<bool(std::shared_ptr<NE>)> const & matchFunction) const {
+   std::shared_ptr<NE> findFirstMatching(std::function<bool(std::shared_ptr<NE>)> const & matchFunction) const {
       //
       // Caller has provided us with a lambda function that takes a shared pointer to NE (ie Water, Hop, Yeast, Recipe,
       // etc) and returns true or false depending on whether it's a match for whatever condition the caller requires.
@@ -211,10 +215,10 @@ public:
       auto result = this->ObjectStore::findFirstMatching(
          [matchFunction](std::shared_ptr<QObject> obj) {return matchFunction(std::static_pointer_cast<NE>(obj));}
       );
-      if (!result.has_value()) {
-         return std::nullopt;
-      }
-      return std::optional< std::shared_ptr<NE> >{std::static_pointer_cast<NE>(result.value())};
+///      if (!result) {
+///         return nullptr;
+///      }
+      return std::shared_ptr<NE>{std::static_pointer_cast<NE>(result)};
    }
 
    /**
@@ -237,10 +241,10 @@ public:
       auto result = this->ObjectStore::findFirstMatching(
          [matchFunction](std::shared_ptr<QObject> obj) {return matchFunction(static_cast<NE *>(obj.get()));}
       );
-      if (!result.has_value()) {
+      if (!result) {
          return nullptr;
       }
-      return static_cast<NE *>(result.value().get());
+      return static_cast<NE *>(result.get());
    }
 
    /**
@@ -278,6 +282,15 @@ public:
       );
    }
 
+   /**
+    * \brief Similary to \c findAllMatching but returns a list of IDs
+    */
+   QVector<int> idsOfAllMatching(std::function<bool(NE const *)> const & matchFunction) const {
+      return this->ObjectStore::idsOfAllMatching(
+         [matchFunction](QObject const * obj) { return matchFunction(static_cast<NE const *>(obj)); }
+      );
+   }
+
    /**
     * \brief Special case of \c findAllMatching that returns a list of all cached objects of a given type
     */
@@ -392,6 +405,28 @@ private:
    ObjectStoreTyped& operator=(ObjectStoreTyped&& other) = delete;
 };
 
+/**
+ * \brief Ensure all the object stores (ie all instances of \c ObjectStoreTyped) are initialised and have read in their
+ *        data from the DB.
+ *
+ *        Although things will usually work fine if we use "lazy loading" (ie let each \c ObjectStoreTyped instance read
+ *        in all its records the first time \c getInstance is called), it is safer to explicitly ask all the instances
+ *        to load their data soon after the \c Database object is initialised.  This is because, if there is a problem
+ *        reading in data from one or more of the tables, we want to tell the user and, most likely, terminate the
+ *        program.  (Continuing on in the face of errors is likely to result in null pointer errors are the program
+ *        tries to reference data it was unable to read out of the DB.)
+ *
+ *        NOTE: It doesn't matter if some or all of the \c ObjectStoreTyped instances were initialised before this
+ *              function was called.  They will not be initialised twice.  If any failed initialisation then that WILL
+ *              be picked up by this function and reported as an error.
+ *
+ * \param errorMessage OUT - In the event of an error, will hold info suitable for showing to the user about which
+ *                           stores could not be initialised.
+ *
+ * \return \c true if everything succeeded, \c false otherwise
+ */
+bool InitialiseAllObjectStores(QString & errorMessage);
+
 /**
  * \brief Does what it says on the tin.  Note that it is the caller's responsibility to handle transactions.
  *
