@@ -1,21 +1,18 @@
-/*
- * database/ObjectStoreWrapper.h is part of Brewtarget, and is copyright the
- * following authors 2021-2022:
+/*╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌
+ * database/ObjectStoreWrapper.h is part of Brewtarget, and is copyright the following authors 2021-2024:
  *   • Matt Young <mfsy@yahoo.com>
  *
- * Brewtarget is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ * Brewtarget is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
+ * version.
  *
- * Brewtarget is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * Brewtarget is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
+ * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ * details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+ * You should have received a copy of the GNU General Public License along with this program.  If not, see
+ * <http://www.gnu.org/licenses/>.
+ ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌*/
 #ifndef DATABASE_OBJECTSTOREWRAPPER_H
 #define DATABASE_OBJECTSTOREWRAPPER_H
 #pragma once
@@ -75,6 +72,7 @@ namespace ObjectStoreWrapper {
       return ObjectStoreTyped<NE>::getInstance().getAll();
    }
 
+   //! \brief Raw pointer version
    template<class NE> QList<NE *> getAllRaw() {
       return ObjectStoreTyped<NE>::getInstance().getAllRaw();
    }
@@ -85,6 +83,13 @@ namespace ObjectStoreWrapper {
     *          - not marked deleted
     *          - do not have a parent (ie are not "an instance of use of"
     */
+   template<class NE> QList<std::shared_ptr<NE>> getAllDisplayable() {
+      return ObjectStoreTyped<NE>::getInstance().findAllMatching(
+         [](std::shared_ptr<NE> ne) { return (ne->display() && !ne->deleted() && ne->getParentKey() <= 0); }
+      );
+   }
+
+   //! \brief Raw pointer version
    template<class NE> QList<NE *> getAllDisplayableRaw() {
       return ObjectStoreTyped<NE>::getInstance().findAllMatching(
          [](NE const * ne) { return (ne->display() && !ne->deleted() && ne->getParentKey() <= 0); }
@@ -190,10 +195,10 @@ namespace ObjectStoreWrapper {
     *
     * \param matchFunction Takes a pointer to an object and returns \c true if the object is a match or \c false otherwise.
     *
-    * \return Shared pointer to the first object that gives a \c true result to \c matchFunction, or \c std::nullopt if
-    *         none does
+    * \return Shared pointer to the first object that gives a \c true result to \c matchFunction, or \c nullptr if none
+    *         does.
     */
-   template<class NE> std::optional< std::shared_ptr<NE> > findFirstMatching(
+   template<class NE> std::shared_ptr<NE> findFirstMatching(
       std::function<bool(std::shared_ptr<NE>)> const & matchFunction
    ) {
       return ObjectStoreTyped<NE>::getInstance().findFirstMatching(matchFunction);
@@ -223,6 +228,11 @@ namespace ObjectStoreWrapper {
       return ObjectStoreTyped<NE>::getInstance().findAllMatching(matchFunction);
    }
 
+   template<class NE>
+   QVector<int> idsOfAllMatching(std::function<bool(NE const *)> const & matchFunction) {
+      return ObjectStoreTyped<NE>::getInstance().idsOfAllMatching(matchFunction);
+   }
+
    /**
     * \brief Given two IDs of some subclass of \c NamedEntity, return \c true if the corresponding objects are equal (or
     *        if both IDs are invalid), and \c false otherwise
